package sun.management;

import javax.management.MBeanNotificationInfo;
import javax.management.Notification;
import javax.management.ListenerNotFoundException;
import java.util.ArrayList;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import java.util.Collections;
import java.util.List;
import javax.management.NotificationEmitter;

abstract class NotificationEmitterSupport implements NotificationEmitter
{
    private Object listenerLock;
    private List<NotificationEmitterSupport.ListenerInfo> listenerList;
    
    protected NotificationEmitterSupport() {
        this.listenerLock = new Object();
        this.listenerList = Collections.emptyList();
    }
    
    @Override
    public void addNotificationListener(NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) {
        if (notificationListener == null) {
            throw new IllegalArgumentException("Listener can't be null");
        }
        synchronized (this.listenerLock) {
            ArrayList<NotificationEmitterSupport.ListenerInfo> listenerList = new ArrayList(this.listenerList.size() + 1);
            listenerList.addAll(this.listenerList);
            listenerList.add(new NotificationEmitterSupport.ListenerInfo(notificationListener, notificationFilter, o));
            this.listenerList = listenerList;
        }
        return;
    }
    
    @Override
    public void removeNotificationListener(NotificationListener notificationListener) throws ListenerNotFoundException {
        synchronized (this.listenerLock) {
            ArrayList listenerList = new ArrayList(this.listenerList);
            for (int i = listenerList.size() - 1; i >= 0; --i) {
                if (((NotificationEmitterSupport.ListenerInfo)listenerList.get(i)).listener == notificationListener) {
                    listenerList.remove(i);
                }
            }
            if (listenerList.size() == this.listenerList.size()) {
                throw new ListenerNotFoundException("Listener not registered");
            }
            this.listenerList = listenerList;
        }
    }
    
    @Override
    public void removeNotificationListener(NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) throws ListenerNotFoundException {
        boolean n = false;
        synchronized (this.listenerLock) {
            ArrayList listenerList = new ArrayList(this.listenerList);
            for (int size = listenerList.size(), i = 0; i < size; ++i) {
                NotificationEmitterSupport.ListenerInfo listenerInfo = (NotificationEmitterSupport.ListenerInfo)listenerList.get(i);
                if (listenerInfo.listener == notificationListener) {
                    n = true;
                    if (listenerInfo.filter == notificationFilter && listenerInfo.handback == o) {
                        listenerList.remove(i);
                        this.listenerList = listenerList;
                        return;
                    }
                }
            }
        }
        if (!(n == 0)) {
            throw new ListenerNotFoundException("Listener not registered with this filter and handback");
        }
        throw new ListenerNotFoundException("Listener not registered");
    }
    
    void sendNotification(Notification notification) {
        if (notification == null) {
            return;
        }
        List<Object> listenerList;
        synchronized (this.listenerLock) {
            listenerList = this.listenerList;
        }
        for (int size = listenerList.size(), i = 0; i < size; ++i) {
            NotificationEmitterSupport.ListenerInfo listenerInfo = (NotificationEmitterSupport.ListenerInfo)listenerList.get(i);
            if (listenerInfo.filter != null && !listenerInfo.filter.isNotificationEnabled(notification)) {
                continue;
            }
            try {
                listenerInfo.listener.handleNotification(notification, listenerInfo.handback);
            }
            catch (Exception ex) {
                ex.printStackTrace();
                throw new AssertionError("Error in invoking listener");
            }
        }
        return;
    }
    
    boolean hasListeners() {
        synchronized (this.listenerLock) {
            return !this.listenerList.isEmpty();
        }
    }
    
    @Override
    public abstract MBeanNotificationInfo[] getNotificationInfo();
    
    private class ListenerInfo
    {
        public NotificationListener listener;
        NotificationFilter filter;
        Object handback;
        
        public ListenerInfo(NotificationListener listener, NotificationFilter filter, Object handback) {
            this.listener = listener;
            this.filter = filter;
            this.handback = handback;
        }
    }
}
